/*

This code is a modified version of MinIMU-9-Arduino-AHRS code which has been modified by Rob and William


 MinIMU-9-Arduino-AHRS
 Pololu MinIMU-9 + Arduino AHRS (Attitude and Heading Reference System)
 
 Copyright (c) 2011 Pololu Corporation.
 http://www.pololu.com/
 
 MinIMU-9-Arduino-AHRS is based on sf9domahrs by Doug Weibel and Jose Julio:
 http://code.google.com/p/sf9domahrs/
 
 sf9domahrs is based on ArduIMU v1.5 by Jordi Munoz and William Premerlani, Jose
 Julio and Doug Weibel:
 http://code.google.com/p/ardu-imu/
 
 MinIMU-9-Arduino-AHRS is free software: you can redistribute it and/or modify it
 under the terms of the GNU Lesser General Public License as published by the
 Free Software Foundation, either version 3 of the License, or (at your option)
 any later version.
 
 MinIMU-9-Arduino-AHRS is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for
 more details.
 
 You should have received a copy of the GNU Lesser General Public License along
 with MinIMU-9-Arduino-AHRS. If not, see <http://www.gnu.org/licenses/>.
 
 */


#include <SoftwareSerial.h>
#include <SabertoothSimplified.h>
#include <Constrain.h>


 // This sample uses a baud rate of 9600.
//
// Connections to make:
// Arduino TX->11 -> Sabertooth S1
// Arduino GND -> Sabertooth 0V
// Arduino VIN -> Sabertooth 5V (OPTIONAL, if you want the Sabertooth to power the Arduino)



// Uncomment the below line to use this axis definition: 
// X axis pointing forward
// Y axis pointing to the right 
// and Z axis pointing down.
// Positive pitch : nose up
// Positive roll : right wing down
// Positive yaw : clockwise
int SENSOR_SIGN[9] = {
  1,1,1,-1,-1,-1,1,1,1}; //Correct directions x,y,z - gyro, accelerometer, magnetometer
// Uncomment the below line to use this axis definition: 
// X axis pointing forward
// Y axis pointing to the left 
// and Z axis pointing up.
// Positive pitch : nose down
// Positive roll : right wing down
// Positive yaw : counterclockwise
//int SENSOR_SIGN[9] = {1,-1,-1,-1,1,1,1,-1,-1}; //Correct directions x,y,z - gyro, accelerometer, magnetometer

// tested with Arduino Uno with ATmega328 and Arduino Duemilanove with ATMega168

#include <Wire.h>

// LSM303 accelerometer: 8 g sensitivity
// 3.8 mg/digit; 1 g = 256
#define GRAVITY 256  //this equivalent to 1G in the raw data coming from the accelerometer 

#define ToRad(x) ((x)*0.01745329252)  // *pi/180
#define ToDeg(x) ((x)*57.2957795131)  // *180/pi

// L3G4200D gyro: 2000 dps full scale
// 70 mdps/digit; 1 dps = 0.07
#define Gyro_Gain_X 0.07 //X axis Gyro gain
#define Gyro_Gain_Y 0.07 //Y axis Gyro gain
#define Gyro_Gain_Z 0.07 //Z axis Gyro gain
#define Gyro_Scaled_X(x) ((x)*ToRad(Gyro_Gain_X)) //Return the scaled ADC raw data of the gyro in radians for second
#define Gyro_Scaled_Y(x) ((x)*ToRad(Gyro_Gain_Y)) //Return the scaled ADC raw data of the gyro in radians for second
#define Gyro_Scaled_Z(x) ((x)*ToRad(Gyro_Gain_Z)) //Return the scaled ADC raw data of the gyro in radians for second

// LSM303 magnetometer calibration constants; use the Calibrate example from
// the Pololu LSM303 library to find the right values for your board
#define M_X_MIN 477
#define M_Y_MIN -708
#define M_Z_MIN 256
#define M_X_MAX 633
#define M_Y_MAX 280
#define M_Z_MAX 446

#define Kp_ROLLPITCH 0.02
#define Ki_ROLLPITCH 0.00002
#define Kp_YAW 1.2
#define Ki_YAW 0.00002

/*For debugging purposes*/
//OUTPUTMODE=1 will print the corrected data, 
//OUTPUTMODE=0 will print uncorrected data of the gyros (with drift)
#define OUTPUTMODE 1

//#define PRINT_DCM 0     //Will print the whole direction cosine matrix
#define PRINT_ANALOGS 0 //Will print the analog raw data
#define PRINT_EULER 1   //Will print the Euler angles Roll, Pitch and Yaw


#define STATUS_LED 13 

float G_Dt=0.02;    // Integration time (DCM algorithm)  We will run the integration loop at 50Hz if possible

long timer=0;   //general purpuse timer
long timer_old;
long timer24=0; //Second timer used to print values 
int AN[6]; //array that stores the gyro and accelerometer data
int AN_OFFSET[6]={
  0,0,0,0,0,0}; //Array that stores the Offset of the sensors


int gyro_x;
int gyro_y;
int gyro_z;
int accel_x;
int accel_y;
int accel_z;
int magnetom_x;
int magnetom_y;
int magnetom_z;
float c_magnetom_x;
float c_magnetom_y;
float c_magnetom_z;
float MAG_Heading;

float Accel_Vector[3]= {
  0,0,0}; //Store the acceleration in a vector
float Gyro_Vector[3]= {
  0,0,0};//Store the gyros turn rate in a vector
float Omega_Vector[3]= {
  0,0,0}; //Corrected Gyro_Vector data
float Omega_P[3]= {
  0,0,0};//Omega Proportional correction
float Omega_I[3]= {
  0,0,0};//Omega Integrator
float Omega[3]= {
  0,0,0};

// Euler angles
float roll;
float pitch;
float yaw;

float errorRollPitch[3]= {
  0,0,0}; 
float errorYaw[3]= {
  0,0,0};

unsigned int counter=0;
byte gyro_sat=0;

const int selectPin = 2; //digital currently not being used
const int joystick_xPin = A0;
const int joystick_yPin = A1; //the one we use
int oldX = 0;
int oldY = 0;
int oldSelect = 0;
int gyroangle;

SoftwareSerial SWSerial(NOT_A_PIN, 11); // RX on no pin (unused), TX on pin 11 (to S1).
SabertoothSimplified ST(SWSerial); // Use SWSerial as the serial port.

float DCM_Matrix[3][3]= {
  {
    1,0,0    }
  ,{
    0,1,0    }
  ,{
    0,0,1    }
}; 
float Update_Matrix[3][3]={
  {
    0,1,2  }
  ,{
    3,4,5  }
  ,{
    6,7,8  }
}; //Gyros here


float Temporary_Matrix[3][3]={
  {
    0,0,0    }
  ,{
    0,0,0    }
  ,{
    0,0,0    }
};

void setup()
{ 
  pinMode(selectPin, INPUT);
  digitalWrite(selectPin, HIGH);
  Serial.begin(9600);
  pinMode (STATUS_LED,OUTPUT);  // Status LED
  pinMode(11, OUTPUT);
  SWSerial.begin(9600);
 //int gyroangle = roll; //(need to find out how it reads the gyro data and computes the angles the read that here...)
  
  I2C_Init();

  Serial.println("Pololu MinIMU-9 + Arduino AHRS");

  digitalWrite(STATUS_LED,LOW);
  delay(1500);

  Accel_Init();
  Compass_Init();
  Gyro_Init();

  delay(20);

  for(int i=0;i<32;i++)    // We take some readings...
  {
    Read_Gyro();
    Read_Accel();
    for(int y=0; y<6; y++)   // Cumulate values
      AN_OFFSET[y] += AN[y];
    delay(20);
  }

  for(int y=0; y<6; y++)
    AN_OFFSET[y] = AN_OFFSET[y]/32;

  AN_OFFSET[5]-=GRAVITY*SENSOR_SIGN[5];

  //Serial.println("Offset:");
  for(int y=0; y<6; y++)
    Serial.println(AN_OFFSET[y]);

  delay(2000);
  digitalWrite(STATUS_LED,HIGH);

  timer=millis();
  delay(20);
  counter=0;
}

void loop() //Main Loop
{
  int joystick;
  int joystick_x = analogRead(joystick_xPin);
  int joystick_y =analogRead(joystick_yPin);
  int select;
 

  //joystick_x = map(joystick_xPin, 0, 1023, 0, 1023); //map function requires 4 arguements (just FYI)
  joystick = map(joystick_y, 0, 1023, -15, 15);
  
  
  /*select = !digitalRead(selectPin);
   if((oldX != joystick_x) || (oldY != joystick_y) || (oldSelect != select)){
   Serial.print("joystick X: ");
   Serial.print(joystick_x);
   Serial.print(" joystick Y: ");
   Serial.print(joystick_y); 
   if(select){
   Serial.print(" select");
   }
   Serial.println(" ");
   oldX = joystick_x;
   oldY = joystick_y;
   oldSelect = select;
   } */
  delay(10);


  if((millis()-timer)>=20)  // Main loop runs at 50Hz
  {
    counter++;
    timer_old = timer;
    timer=millis();
    if (timer>timer_old)
      G_Dt = (timer-timer_old)/1000.0;    // Real time of loop run. We use this on the DCM algorithm (gyro integration time)
    else
      G_Dt = 0;

    // *** DCM algorithm
    // Data adquisition
    Read_Gyro();   // This read gyro data
    Read_Accel();     // Read I2C accelerometer

    if (counter > 5)  // Read compass data at 10Hz... (5 loop runs)
    {
      counter=0;
      Read_Compass();    // Read I2C magnetometer
      Compass_Heading(); // Calculate magnetic heading  
    }

    // Calculations...
    Matrix_update(); 
    Normalize();
    Drift_correction();
    Euler_angles();
    // ***

    printdata();
    
  }
 // Serial.print(" joystick Y: ");
//Serial.print( joystick); 
    
 
     
  int gyroangle = ToDeg(roll);
  int fpower = ToDeg(roll); //power for forward gyro angle dependant
  int accel = map(fpower, 12, 80, -63.5, 63.5);
  int bpower = -ToDeg(roll); // power for backwards gyro angle dependant
  int fpower_add = accel + joystick;
  int fpower_sub = accel - joystick;
  int bpower_add = accel - joystick;
  int bpower_sub = accel + joystick;
  constrain(fpower, -63.5, 63.5); //constraining power to half of motor's full output (-127 to 127 is full output)
  constrain(bpower, -63.5, 0); //constraining power to half of motor's full output (-127 to 127 is full output)
  constrain(fpower_add, 0, 63.5);
  constrain(bpower_sub, -63.5, 0);
  constrain(bpower_add, -63.5, 0);
  constrain(accel, -63.5, 63.5);
  
  Serial.print ("DCM is: ");
  Serial.print (ToDeg(roll));
 /*
    //Serial.print ("gyroangle is ");
   // Serial.print (gyroangle);
    Serial.print("accel is ");
    Serial.print( accel); 
    Serial.print(" fpower  add");
    Serial.print( fpower_add); 
    Serial.print(" fpower sub ");
    Serial.print( fpower_sub); 
    Serial.print(" bpower add ");
    
    Serial.print( bpower_add); 
    Serial.print(" bpower sub  ");
    Serial.print( bpower_sub); 
    Serial.print("bpower is ");
    Serial.print( bpower);
   */
    if (gyroangle < 38 && joystick == 0) {//-4.6
   //go in varied reverse....have to look at angle data from gyro to determine
         ST.motor(1, accel);//MR go reverse (some range between 0 and -127 which is dependant upon angle)
         ST.motor(2, accel); //ML go reverse (subtracting from negative makes larger negative)
         Serial.print (" reverse ");
         delay(20);
         }
   if (gyroangle < 38 && joystick > 0) {
         //go in varied reverse...I just choose 2 numbers....have to look at angle data from gyro to determine
         ST.motor(1, bpower_add);//MR go reverse
         ST.motor(2, bpower_sub);//ML go reverse
         Serial.print (" rev right ");
         delay(20);
         //x > 0 means go right
         }
    if (gyroangle < 38 &&  joystick < 0) {
        // go in varied reverse...I just choose 2 numbers....have to look at angle data from gyro to determine
         ST.motor(1, bpower_sub);//MR go reverse 
         ST.motor(2, bpower_add);// ML go reverse
         Serial.print (" rev left ");
         delay(20);
        // x < 0 means go left
         }      
 
   if (gyroangle > 38 && gyroangle < 40 && joystick == 0) { //-4.6 
     //you are balanced stay still...I am thinking maybe just let freewill...but will play with this)
         ST.motor(1, 0);//MR do nothing
         ST.motor(2, 0);// ML do nothing
         Serial.print (" do nothing ");
         delay(20);
         }
     if (gyroangle > 38 && gyroangle < 40 && joystick > 0) {
         //turn right but stay balanced...goign to have to play with this portion
         ST.motor(1, (-joystick));// MR slow right  ** map so it is just turning you
         ST.motor(2, joystick);//ML speed left   ** map so it is just turning you
         Serial.print (" go right ");
         delay(20);
        // x > 0 means go right
         } 
      if (gyroangle > 38 && gyroangle < 40 && joystick < 0) {
         //go in varied reverse...I just choose 2 numbers....have to look at angle data from gyro to determine
        ST.motor(1, joystick); //MR go reverse   ** map so it is just turning you
        ST.motor(2, (-joystick)); //ML go reverse   ** map so it is just turning you
        Serial.print (" go left ");
         delay(20);
         //x < 0 means go left
         }
     
    if (gyroangle > 40 && joystick == 0) { //-4.6
 
        ST.motor(1, accel); //MR go forward
        ST.motor(2, accel); //ML go forward
        Serial.print (" forward ");
        delay(20);
        }
  if (gyroangle > 40 && joystick > 0) {
        //go in varied reverse...I just choose 2 numbers....have to look at angle data from gyro to determine
        ST.motor(1, fpower_sub); //MR go forward
        ST.motor(2, fpower_add); //ML go forward
        Serial.print (" forward right ");
        delay(20);
        //x > 0 means go right
        }
   if (gyroangle > 40 && joystick < 0) {
        //go in varied reverse...I just choose 2 numbers....have to look at angle data from gyro to determine
        ST.motor(1, fpower_add); //MR go forward
        ST.motor(2, fpower_sub); //ML go forward
        Serial.print (" forward left ");
        delay(20);
        //x < 0 means go left
       }
   

}

